
// Package name:
package org.samplethesaurus;

import java.util.*;
import java.io.*;

public class SampleThesaurus {
    
    // Have we made one pass to extract location info?
    static boolean loaded = false;

    // Gives us the line# in thesaurus.txt for a particular word.
    static HashMap<String, Integer> locationIndex;

    // This is the number of entries per word.
    static HashMap<String, Integer> linesToRead;

    // The file thesaurus.txt
    static File mainFile = null;

    // For every category, the type:
    static HashMap<Integer, String> categoryType;

    // Words organized by category.
    static HashMap<Integer, ArrayList<String>> categoryList;


    // This main() is only for testing.

    public static void main (String[] argv)
    {
	openThesaurus ();
	int numCategories = search ("pick");
	System.out.println ("numCat found = " + numCategories);
	for (int i=0; i<numCategories; i++) {
	    String type = getTypeName (i);
	    ArrayList<String> syns = getSynonyms (i);
	    System.out.print ("Type #" + i + ": " + type + ": ");
	    for (String w: syns) {
		System.out.print (" | " + w);
	    }
	    System.out.println ();
	}
    }

    public static boolean openThesaurus ()
    {
	loaded = false;
	locationIndex = new HashMap<String,Integer>();
	linesToRead = new HashMap<String,Integer>();

	try {
	    // Where is the thesaurus?
	    String thesHome = System.getenv ("THESAURUS_HOME");

	    if (thesHome==null) {
		System.out.println ("ERROR: thesaurus env variable not set");
		return false;
	    }

	    File f = new File (thesHome);
	    if (! f.isDirectory()) {
		System.out.println ("ERROR: cannot find thesaurus directory");
		return false;
	    }

	    mainFile = new File (thesHome + File.separatorChar + "thesaurus.txt");

	    LineNumberReader lnr = new LineNumberReader (new FileReader (mainFile));
	    // Skip first line
	    String line = lnr.readLine ();
	    line = lnr.readLine ();

	    int k = 1;
	    while (line != null) {
		int linesToSkip = process (line, k);
		// At this time, we only process the entry and
		// not the synonyms.
		for (int i=0; i<linesToSkip; i++) {
		    line = lnr.readLine ();
		    k++;
		}
		line = lnr.readLine ();
		k++;
		//if (k >= 1000) break;
	    }

	    loaded = true;
	    return true;
	}
	catch (Exception e) {
	    System.out.println ("ERROR: cannot load thesaurus");
	}
	return false;
    }

    public static int search (String word)
    {
	if (! loaded) {
	    return 0;
	}

	try {
	    // See if it exists in the index.
	    Integer location = locationIndex.get (word);
	    if (location == null) {
		return 0;
	    }

	    // Now we have the line number.
	    LineNumberReader lnr = new LineNumberReader (new FileReader (mainFile));
	    String line = lnr.readLine ();
	    int k = 0;
	    while (k < location) {
		line = lnr.readLine ();
		k++;
	    }

	    Integer numLines = linesToRead.get (word);
	    String[] lines = new String [numLines];
	    for (int i=0; i<numLines; i++) {
		lines[i] = lnr.readLine ();
	    }

	    // Extract all the info for the given word.
	    extract (word, lines);

	    // This is the number categories:
	    return numLines;
	}
	catch (Exception e) {
	}
	return 0;
    }


    public static String getTypeName (int typeNum)
    {
	return categoryType.get(typeNum);
    }


    public static ArrayList<String> getSynonyms (int typeNum)
    {
	return categoryList.get(typeNum);
    }


    static void extract (String word, String[] lines)
	throws Exception                 
    {
	categoryType = new HashMap<Integer, String> ();
	categoryList = new HashMap<Integer, ArrayList<String>> ();

	for (int i=0; i<lines.length; i++) {
	    String[] parts = lines[i].split ("\\|");

	    // Remove parens from type:
	    String type = parts[0].substring (1, parts[0].length()-1);

	    categoryType.put (i, type);

	    // The remaining parts are actual synonyms:
	    ArrayList<String> syns = new ArrayList<String> ();
	    for (int j=1; j<parts.length; j++) {
		syns.add (parts[j].trim());
	    }
	    categoryList.put (i, syns);
	}
    }


    static int process (String line, int lineNum)
	throws Exception                 // Number format etc.
    {
	// The parameter line should be a real entry. The
	// number after the | tells us how many added lines there are.
	String[] parts = line.split ("\\|");

	// There should only be 2 parts.
	int numLines = Integer.parseInt (parts[1].trim());
	locationIndex.put (parts[0], lineNum);
	linesToRead.put (parts[0], numLines);

	return numLines;
    }


}
